/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.presenter;

import java.util.Set;

/**
 * Interface for UI Model and Presenter
 * 
 * @since v1.0
 * @author Sam Elsharif
 *
 */
public interface LauncherPresenter
{
	/**
	 * Starts the UI application
	 */
	void start();

	/**
	 * Called when user requests settings screen (e.g. by clicking on settings menu item)
	 */
	void onRequestSettingsScreen();

	/**
	 * Called when user accepts settings (e.g. by clicking on OK button in settings screen)
	 * 
	 * @param openMSXFullPath Full path to directory of openMSX binary
	 * @param openMSXMachinesFullPath Full path to openMSX machines directory
	 * @param screenshotsFullPath Full path to screenshots directory
	 * @param defaultDatabase Default database
	 * @param languageChoice String that represents language name in Language enum
	 * @throws LauncherException
	 */
	void onRequestSettingsAction( String openMSXFullPath,
			String openMSXMachinesFullPath,
			String screenshotsFullPath,
			String defaultDatabase,
			String languageChoice ) throws LauncherException;

	/**
	 * Called when user selects a database to view
	 * 
	 * @param database Selected database name
	 * @throws LauncherException
	 */
	void onSelectDatabase( String database ) throws LauncherException;

	/**
	 * Called when user selects one or more games
	 * 
	 * @param gameNames Set containing selected game names
	 */
	void onSelectGames( Set<String> gameNames );

	/**
	 * Called when user launches openMSX with selected game
	 * 
	 * @param gameName Name of game to launch
	 * @throws LauncherException
	 */
	void onLaunchGame( String gameName ) throws LauncherException;

	/**
	 * Called when user requests the add game screen (e.g. by clicking on the Add Game button)
	 */
	void onRequestAddGameScreen() throws LauncherException;

	/**
	 * Called when user requests the edit game screen (e.g. by clicking on the Edit Game button)
	 * 
	 * @param gameName Name of game to edit
	 * @throws LauncherException
	 */
	void onRequestEditGameScreen( String gameName ) throws LauncherException;

	/**
	 * Called when user requests to save the game from within the Add screen
	 * 
	 * @param name Name of game
	 * @param info Full path to info file or URL for a game
	 * @param machine Machine name
	 * @param romsUsed Flag to indicate if a ROM not an extension is used
	 * @param romA Full path to ROM A file
	 * @param romB Full path to ROM B file
	 * @param extensionRom Name of openMSX extension
	 * @param diskA Full path to Disk A file
	 * @param diskB Full path to Disk B file
	 * @param tape Full path to Tape file
	 * @param harddisk Full path to Hard Disk file
	 * @param laserdisc Full path to Laserdisc file
	 * @param script Full path to Script file
	 * @throws LauncherException
	 */
	void onRequestAddGameSaveAction( String name,
			String info,
			String machine,
			boolean romsUsed,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script) throws LauncherException;

	/**
	 * Called when user decides to launch the game from within the Add or Edit screens
	 * 
	 * @param machine Machine name
	 * @param romsUsed Flag to indicate if a ROM not an extension is used
	 * @param romA Full path to ROM A file
	 * @param romB Full path to ROM B file
	 * @param extensionRom Name of openMSX extension
	 * @param diskA Full path to Disk A file
	 * @param diskB Full path to Disk B file
	 * @param tape Full path to Tape file
	 * @param harddisk Full path to Hard Disk file
	 * @param laserdisc Full path to Laserdisc file
	 * @param script Full path to Script file
	 * @throws LauncherException
	 */
	void onRequestAddEditGameLaunchAction( String machine,
			boolean romsUsed,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script) throws LauncherException;

	/**
	 * Called when user requests to save the game from within the Edit screen
	 * 
	 * @param oldName Current name of game to be edited
	 * @param newName New name for the game to be edited - can be the same as the old name
	 * @param info Full path to info file or URL for a game
	 * @param machine Machine name
	 * @param romsUsed Flag to indicate if a ROM not an extension is used
	 * @param romA Full path to ROM A file
	 * @param romB Full path to ROM B file
	 * @param extensionRom Name of openMSX extension
	 * @param diskA Full path to Disk A file
	 * @param diskB Full path to Disk B file
	 * @param tape Full path to Tape file
	 * @param harddisk Full path to Hard Disk file
	 * @param laserdisc Full path to Laserdisc file
	 * @param script Full path to Script file
	 * @throws LauncherException
	 */
	void onRequestEditGameSaveAction( String oldName,
			String newName,
			String info,
			String machine,
			boolean romsUsed,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script ) throws LauncherException;

	/**
	 * Called when user requests to remove selected games
	 * 
	 * @param gameNames
	 * @param database
	 * @throws LauncherException
	 */
	void onRequestRemoveGamesAction( Set<String> gameNames, String database ) throws LauncherException;

	/**
	 * Called when user requests fill database screen (e.g. by clicking on fill database menu item)
	 */
	void onRequestFillDatabaseScreen() throws LauncherException;

	/**
	 * Called when user requests to fill a database with the given parameters
	 * 
	 * @param paths
	 * @param traverseSubDirectories
	 * @param database
	 * @param newDatabase
	 * @param append
	 * @param machine
	 * @param searchROM
	 * @param searchDisk
	 * @param searchTape
	 * @param searchLaserdisc
	 * @param getNameFromOpenMSXDatabase
	 * @param backupDatabase
	 * @return
	 * @throws LauncherException
	 */
	int onRequestFillDatabaseAction( String[] paths,
			boolean traverseSubDirectories,
			String database,
			boolean newDatabase,
			boolean append,
			String machine,
			boolean searchROM,
			boolean searchDisk,
			boolean searchTape,
			boolean searchLaserdisc,
			boolean getNameFromOpenMSXDatabase,
			boolean backupDatabase ) throws LauncherException;

	/**
	 * Called when it is needed to update the game list in the current database (e.g. after running fill database)
	 * 
	 * @param database Name of current database
	 * @throws LauncherException
	 */
	void onUpdateViewedDatabase( String database ) throws LauncherException;

	/**
	 * Called when user requests to interrupt the running fill database process
	 */
	void onRequestInterruptFillDatabaseProcess();

	/**
	 * Called when user requests view the game's info file, or web URL in a browser
	 * 
	 * @param gameName Name of game
	 * @throws LauncherException
	 */
	void onRequestGameInfo( String gameName ) throws LauncherException;

	/**
	 * Called when user requests the About screen
	 */
	void onRequestAboutScreen();

	/**
	 * Called when user requests the move games screen
	 * 
	 * @param gameNames Names of games to move
	 * @param oldDatabase Name of the database to move the selected games from
	 * @return Set containing games that were moved successfully
	 */
	Set<String> onRequestMoveGamesScreen( Set<String> gameNames, String oldDatabase );

	/**
	 * Called when user requests the move games action (e.g. by clicking OK in the move games screen after selecting target database
	 * 
	 * @param gameNames Names of games to move
	 * @param oldDatabase Name of the database to move the selected games from
	 * @param newDatabase Name of the target database to move the selected games to
	 * @return Set containing games that were moved successfully
	 * @throws LauncherException
	 */
	Set<String> onRequestMoveGamesAction( Set<String> gameNames, String oldDatabase, String newDatabase ) throws LauncherException;

	/**
	 * Called when user requests the create empty database screen
	 */
	void onRequestCreateEmptyDatabaseScreen();

	/**
	 * Called when user requests to create an empty database with the given name
	 * 
	 * @param newDatabase Name of the new empty database
	 * @throws LauncherException
	 */
	void onRequestCreateEmptyDatabaseAction( String newDatabase ) throws LauncherException;

	/**
	 * Called when user requests the help file
	 * 
	 * @throws LauncherException
	 */
	void onRequestHelpFile() throws LauncherException;

	/**
	 * Called when user requests to locate the main file of the selected game
	 * 
	 * @param gameName Name of game to locate its main file
	 * @throws LauncherException
	 */
	void onRequestLocateFile( String gameName ) throws LauncherException;
}
