/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.presenter;

import info.msxlaunchers.openmsx.common.FileTypeUtils;
import info.msxlaunchers.openmsx.common.HashUtils;
import info.msxlaunchers.openmsx.common.IOUtils;
import info.msxlaunchers.openmsx.common.Utils;
import info.msxlaunchers.openmsx.extension.ExtensionLister;
import info.msxlaunchers.openmsx.game.RepositoryGame;
import info.msxlaunchers.openmsx.game.repository.RepositoryData;
import info.msxlaunchers.openmsx.game.scan.Scanner;
import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.data.global.Settings;
import info.msxlaunchers.openmsx.launcher.data.global.constants.Language;
import info.msxlaunchers.openmsx.launcher.extra.ExtraData;
import info.msxlaunchers.openmsx.launcher.extra.ExtraDataGetter;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseMaxBackupReachedException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNullNameException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameWithNullNameException;
import info.msxlaunchers.openmsx.launcher.persistence.settings.SettingsPersister;
import info.msxlaunchers.openmsx.launcher.starter.EmulatorStarter;
import info.msxlaunchers.openmsx.launcher.ui.view.LauncherView;
import info.msxlaunchers.openmsx.machine.InvalidMachinesDirectoryException;
import info.msxlaunchers.openmsx.machine.MachineLister;
import info.msxlaunchers.platform.FileLocator;

import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import com.google.inject.Inject;

/**
 * Implementation of <code>LauncherPresenter</code>
 * 
 * @since v1.0
 * @author Sam Elsharif
 *
 */
final class LauncherPresenterImpl implements LauncherPresenter
{
	private final LauncherView view;
	private final SettingsPersister settingsPersister;
	private final GamePersister gamePersister;
	private final EmulatorStarter emulatorStarter;
	private final Scanner scanner;
	private final MachineLister machineLister;
	private final ExtensionLister extensionLister;
	private final Map<String,ExtraData> extraDataMap;
	private final String helpFileDirectory;
	private final RepositoryData repositoryData;
	private final FileLocator fileLocator;

	private Settings settings;

	private Language currentLanguage;
	private String currentLanguageCode;
	private boolean currentRightToLeft;

	private Set<String> databases = null;
	private String currentDatabase = null;;
	private Map<String,Game> gamesMap = null;
	private String openMSXMachinesFullPath = null;
	private Map<String,RepositoryGame> repositoryInfoMap;

	private static final String SCREENSHOT_EXT = ".png";
	private static final String SCREENSHOT1_SUFFIX = "a";
	private static final String SCREENSHOT2_SUFFIX = "b";

	private static final String DEFAULT_SYSTEM_LANGUAGE = "SYSTEM_DEFAULT";
	private static final String EMPTY_STRING = "";

	@Inject
	LauncherPresenterImpl( LauncherView view,
			SettingsPersister settingsPersister,
			GamePersister gamePersister,
			EmulatorStarter emulatorStarter,
			Scanner scanner,
			MachineLister machineLister,
			ExtensionLister extensionLister,
			ExtraDataGetter extraDataGetter,
			RepositoryData repositoryData,
			FileLocator fileLocator ) throws IOException
	{
		this.view = Utils.notNull( "view", view );
		this.settingsPersister = Utils.notNull( "settingsPersister", settingsPersister );
		this.gamePersister = Utils.notNull( "gamePersister", gamePersister );
		this.emulatorStarter = Utils.notNull( "emulatorStarter", emulatorStarter );
		this.scanner = Utils.notNull( "scanner", scanner );
		this.machineLister = Utils.notNull( "machineLister", machineLister );
		this.extensionLister = Utils.notNull( "extensionLister", extensionLister );
		this.extraDataMap = Utils.notNull( "extraDataGetter", extraDataGetter ).getExtraData();
		this.helpFileDirectory = extraDataGetter.getExtraDataDirectory();
		this.repositoryData = Utils.notNull( "repositoryData", repositoryData );
		this.fileLocator = Utils.notNull( "fileLocator", fileLocator );

		try
		{
			this.settings = settingsPersister.getSettings();
		}
		catch ( IOException ioe )
		{
			//This gets called at application initialisation time, so if it fails then log it and rethrow
			//TODO: add proper logging
			throw ioe;
		}

		setLanguageParameters( settings.getLanguage() );

		databases = gamePersister.getDatabases();
		currentDatabase = settings.getDefaultDatabase();

		if( currentDatabase != null )
		{
			try
			{
				retrieveDatabaseGames( currentDatabase );
			}
			catch ( DatabaseNotFoundException dnfe )
			{
				//This gets called at application initialisation time, so if it fails then just log it
				//TODO: add proper logging
			}
			catch ( IOException ioe )
			{
				//This gets called at application initialisation time, so if it fails then just log it
				//TODO: add proper logging
			}
		}

		openMSXMachinesFullPath = settings.getOpenMSXMachinesFullPath();
		initializeRepositoryInfoMap();

		view.setPresenter( this );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#start()
	 */
	@Override
	public void start()
	{
		view.displayMain( currentLanguage, getSortedGameList(), databases, currentDatabase, currentRightToLeft );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onClickMenuOptionsSettings()
	 */
	@Override
	public void onRequestSettingsScreen()
	{
		Settings oldSettings = this.settings;
		view.displaySettings( oldSettings,
				gamePersister.getDatabases(),
				currentLanguage,
				currentLanguageCode,
				currentRightToLeft );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onClickMenuOptionsSettingsOK(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void onRequestSettingsAction( String openMSXFullPath,
			String openMSXMachinesFullPath,
			String screenshotsFullPath,
			String defaultDatabase,
			String languageChoice )
					throws LauncherException
	{
		try
		{
			Settings oldSettings = this.settings;
			Language language = null;
			if( languageChoice != null )
			{
				language = Language.valueOf( languageChoice );
			}

			Settings newSettings = new Settings( openMSXFullPath,
					openMSXMachinesFullPath,
					screenshotsFullPath,
					defaultDatabase,
					language );

			settingsPersister.saveSettings( newSettings );

			this.settings = newSettings;

			//check if the openMSX path changed so we reload the Repository Info Map
			if( !equalStrings( this.openMSXMachinesFullPath, openMSXMachinesFullPath ) )
			{
				this.openMSXMachinesFullPath = openMSXMachinesFullPath;
				initializeRepositoryInfoMap();
			}

			//only do the following if the a new language was selected
			Language oldLanguage = oldSettings.getLanguage();
			Language newLanguage = settings.getLanguage();

			if( (oldLanguage == null && newLanguage != null) ||
					(oldLanguage != null && newLanguage == null) ||
					(oldLanguage != null && newLanguage != null && !oldLanguage.equals( newLanguage )) )
			{
				boolean oldRightToLeft = this.currentRightToLeft;
				setLanguageParameters( language );
	
				if( oldRightToLeft && Language.isLeftToRight( currentLanguage ) )
				{
					view.flipOrientationLeftToRight();
				}
				else if( Language.isRightToLeft( currentLanguage ) && !oldRightToLeft )
				{
					view.flipOrientationRightToLeft();
				}
				view.refreshLanguage( currentLanguage );
			}
		}
		catch ( IOException e )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_CANNOT_SAVE_SETTINGS );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onSelectDatabase(java.lang.String)
	 */
	@Override
	public void onSelectDatabase( String database ) throws LauncherException
	{
		if( database != null && !database.equals( currentDatabase ) )
		{
			try
			{
				retrieveDatabaseGames( database );
			}
			catch ( DatabaseNotFoundException dnfe )
			{
				throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, database );
			}
			catch ( IOException e )
			{
				throw new LauncherException( LauncherExceptionCode.ERR_IO );
			}

			currentDatabase = database;
			Set<GameLabel> gameLabels = getSortedGameList();
			view.fillGameList( database, gameLabels, null );
			view.updateGameCount( gameLabels.size() );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onLaunchGame(java.lang.String)
	 */
	@Override
	public void onLaunchGame( String gameName ) throws LauncherException
	{
		try
		{
			emulatorStarter.start( settings, gamesMap.get( gameName ) );
		}
		catch ( IOException e )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_CANNOT_START_OPENMSX );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onAddGame()
	 */
	@Override
	public void onRequestAddGameScreen() throws LauncherException
	{
		try
		{
			view.displayAddEdit( currentLanguage,
					machineLister.get(),
					extensionLister.get(),
					Language.isRightToLeft( currentLanguage ) );
		}
		catch( InvalidMachinesDirectoryException imde )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_INVALID_MACHINES_DIRECTORY );
		}
		catch( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onEditGame(java.lang.String)
	 */
	@Override
	public void onRequestEditGameScreen( String gameName ) throws LauncherException
	{
		Game game = gamesMap.get( gameName );
		try
		{
			view.displayAddEdit( currentLanguage,
					machineLister.get(),
					extensionLister.get(),
					Language.isRightToLeft( currentLanguage ),
					game.getName(),
					game.getInfo(),
					game.getMachine(),
					game.getRomA(),
					game.getRomB(),
					game.getExtensionRom(),
					game.getDiskA(),
					game.getDiskB(),
					game.getTape(),
					game.getHarddisk(),
					game.getLaserdisc(),
					game.getTclScript() );
		}
		catch( InvalidMachinesDirectoryException imde )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_INVALID_MACHINES_DIRECTORY );
		}
		catch ( IOException e )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_CANNOT_START_OPENMSX );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onAddGameClickSave(java.lang.String, java.lang.String, java.lang.String, boolean, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void onRequestAddGameSaveAction( String name,
			String info,
			String machine,
			boolean romsUsed,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script) throws LauncherException
	{
		if( romsUsed )
		{
			extensionRom = null;
		}
		else
		{
			romA = null;
			romB = null;
		}

		Game game = null;
		
		try
		{
			game = getGameObject( name, info, machine, romA, romB, extensionRom, diskA, diskB,
				tape, harddisk, laserdisc, script );
		}
		catch( IllegalArgumentException iae )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_EMPTY_GAME_FIELDS );
		}

		if( game == null )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_EMPTY_GAME_FIELDS );
		}

		try
		{
			gamePersister.saveGame( game, currentDatabase );

			gamesMap.put( name,  game );
			Set<GameLabel> gameLabels = getSortedGameList();
			view.fillGameList( currentDatabase, gameLabels, name );
			view.updateGameCount(gameLabels.size());
		}
		catch ( GameWithNullNameException gwnme )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_WITH_NULL_NAME, name );
		}
		catch ( GameAlreadyExistsException gaee )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_ALREADY_EXISTS, name );
		}
		catch ( DatabaseNotFoundException dnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, currentDatabase );
		}
		catch ( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onAddEditGameClickLaunch(java.lang.String, boolean, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void onRequestAddEditGameLaunchAction( String machine,
			boolean romsUsed,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script) throws LauncherException
	{
		if( romsUsed )
		{
			extensionRom = null;
		}

		Game game = null;
		try
		{
			game = Game.machine( machine )
				.romA( romA ).romB( romB ).extensionRom( extensionRom )
				.diskA( diskA ).diskB( diskB )
				.tape( tape )
				.harddisk( harddisk )
				.laserdisc( laserdisc )
				.tclScript( script )
				.build();
		}
		catch ( IllegalArgumentException iae )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_EMPTY_GAME_FIELDS );
		}

		try
		{
			emulatorStarter.start( settings, game );
		}
		catch ( IOException e )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_CANNOT_START_OPENMSX );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onEditGameClickSave(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void onRequestEditGameSaveAction( String oldName,
			String newName,
			String info,
			String machine,
			boolean romsUsed,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script) throws LauncherException
	{
		if( romsUsed )
		{
			extensionRom = null;
		}
		else
		{
			romA = null;
			romB = null;
		}

		Game newGame = null;
		try
		{
			newGame = getGameObject( newName, info, machine, romA, romB, extensionRom, diskA, diskB,
				tape, harddisk, laserdisc, script );
		}
		catch ( IllegalArgumentException iae )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_EMPTY_GAME_FIELDS );
		}

		Game oldGame = gamesMap.get( oldName );

		try
		{
			gamePersister.updateGame( oldGame, newGame, currentDatabase );

			gamesMap.remove( oldName );
			gamesMap.put( newName,  newGame );
			Set<GameLabel> gameLabels = getSortedGameList();
			view.fillGameList( currentDatabase, gameLabels, newName );
			view.updateGameCount(gameLabels.size());
		}
		catch ( GameWithNullNameException gwnne )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_WITH_NULL_NAME );
		}
		catch ( GameNotFoundException gnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_NOT_FOUND, oldName );
		}
		catch ( GameAlreadyExistsException gaee )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_ALREADY_EXISTS, newName );
		}
		catch ( DatabaseNotFoundException dnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, currentDatabase );
		}
		catch ( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onRemoveGames(java.util.Set, java.lang.String)
	 */
	@Override
	public void onRequestRemoveGamesAction( Set<String> gameNames, String database ) throws LauncherException
	{
		//create a Game set from the game names
		Set<Game> games = new HashSet<Game>();

		for( String gameName: gameNames )
		{
			games.add( gamesMap.get( gameName ) );
			gamesMap.remove( gameName );
		}

		try
		{
			gamePersister.deleteGames( games, database );
		}
		catch ( GameNotFoundException gnfe )
		{
			//maybe the game didn't exist - don't really need to process that
		}
		catch ( DatabaseNotFoundException dnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, database );
		}
		catch ( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}

		view.updateGameCount( gamesMap.size() );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onSelectGame(java.util.Set)
	 */
	@Override
	public void onSelectGames( Set<String> gameNames )
	{
		if( gameNames == null || gameNames.size() != 1 )
		{
			//this is the multi selection case
			view.showGameScreenshots( null, null );
			view.enableButtons( false, true, false, false );
			view.resetGameCompanyYearSizeData();
		}
		else
		{
			//this is the single selection case
			Game game = gamesMap.get( gameNames.iterator().next() );

			String screenshot1 = null;
			String screenshot2 = null;

			if( game != null )
			{
				String screenshotsPath = settings.getScreenshotsFullPath();
				if( screenshotsPath != null )
				{
					int msxGenID = game.getMsxGenID();
					if( msxGenID > 0 )
					{
						StringBuilder screenshot1Filename = new StringBuilder( Utils.getString( msxGenID ) )
															.append( SCREENSHOT1_SUFFIX );
						String screenshotSuffix = game.getScreenshotSuffix();
						if( screenshotSuffix != null )
						{
							screenshot1Filename.append( screenshotSuffix );
						}
						screenshot1Filename.append( SCREENSHOT_EXT );

						File screenshot1File = new File( screenshotsPath, screenshot1Filename.toString() );
						if( screenshot1File.isFile() )
						{
							screenshot1 = screenshot1File.toString();
	
							StringBuilder screenshot2Filename = new StringBuilder( Utils.getString( msxGenID ) )
																.append( SCREENSHOT2_SUFFIX );
							if( screenshotSuffix != null )
							{
								screenshot2Filename.append( screenshotSuffix );
							}
							screenshot2Filename.append( SCREENSHOT_EXT );
	
							File screenshot2File = new File( screenshotsPath, screenshot2Filename.toString() );
							if( screenshot2File.isFile() )
							{
								screenshot2 = screenshot2File.toString();
							}
						}
					}
	
					view.showGameScreenshots( screenshot1, screenshot2  );
				}
				else
				{
					view.showGameScreenshots( null, null  );
				}
				
				//enable buttons according to the selection
				view.enableButtons( true,  true, true, game.getInfo() != null );

				//show game details
				String company = EMPTY_STRING;
				String year = EMPTY_STRING;

				if( repositoryInfoMap != null )
				{
					RepositoryGame repositoryGame = repositoryInfoMap.get( game.getSha1Code() );
					if( repositoryGame != null )
					{
						company = repositoryGame.getCompany();
						year = repositoryGame.getYear();
					}
				}
				view.showGameCompanyYearSizeData( company, year, game.getSize() / 1024 );
			}
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onRequestFillDatabase(java.lang.String[], boolean, java.lang.String, boolean, boolean, java.lang.String, boolean, boolean, boolean, boolean, boolean, boolean)
	 */
	@Override
	public int onRequestFillDatabaseAction(String[] paths,
			boolean traverseSubDirectories,
			String database,
			boolean newDatabase,
			boolean append,
			String machine,
			boolean searchROM,
			boolean searchDisk,
			boolean searchTape,
			boolean searchLaserdisc,
			boolean getNameFromOpenMSXDatabase,
			boolean backupDatabase)
					throws LauncherException
	{
		int totalFound = 0;
		try
		{
			//scan and update the database
			totalFound = scanner.scan(paths,
					traverseSubDirectories,
					database,
					newDatabase,
					append,
					machine,
					searchROM,
					searchDisk,
					searchTape,
					searchLaserdisc,
					getNameFromOpenMSXDatabase,
					backupDatabase);
		}
		catch( GameWithNullNameException gwnme )
		{
			//this should not happen because all games we find will have names
		}
		catch( DatabaseNullNameException dnne )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NULL_NAME );
		}
		catch( DatabaseMaxBackupReachedException dmbre )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_MAX_BACKUPS_REACHED, database );
		}
		catch( DatabaseAlreadyExistsException daee )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_ALREADY_EXISTS, database );
		}
		catch( DatabaseNotFoundException dnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, database );
		}
		catch( IOException e )
		{
			//maybe the backup, overwrite, or game saving failed
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}

		return totalFound;
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onUpdateViewedDatabase(java.lang.String)
	 */
	@Override
	public void onUpdateViewedDatabase( String database ) throws LauncherException
	{
		if( !databases.contains( database ) )
		{
			//then this is a new database
			view.addDatabase( database );
			databases.add( database );
		}
		else if( currentDatabase != null && currentDatabase.equals( database ) )
		{
			//then the currently displayed database was modified
			try
			{
				retrieveDatabaseGames( database );
			}
			catch ( DatabaseNotFoundException dnfe )
			{
				throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, database );
			}
			catch ( IOException e )
			{
				throw new LauncherException( LauncherExceptionCode.ERR_IO );
			}

			currentDatabase = database;
			Set<GameLabel> gameLabels = getSortedGameList();
			view.fillGameList( database, gameLabels, null );
			view.updateGameCount(gameLabels.size());
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onInterruptFillDatabaseProcess()
	 */
	@Override
	public void onRequestInterruptFillDatabaseProcess()
	{
		scanner.interrupt();
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onClickMenuActionsFillDatabase()
	 */
	@Override
	public void onRequestFillDatabaseScreen() throws LauncherException
	{
		try
		{
			view.displayFillDatabase( currentLanguage,
					databases,
					currentDatabase,
					machineLister.get(),
					Language.isRightToLeft( currentLanguage ) );
		}
		catch( InvalidMachinesDirectoryException imde )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_INVALID_MACHINES_DIRECTORY );
		}
		catch ( IOException e )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onRequestGameInfo(java.lang.String)
	 */
	@Override
	public void onRequestGameInfo( String gameName ) throws LauncherException
	{
		Game game = gamesMap.get( gameName );

		if( game != null )
		{
			String gameInfo = game.getInfo();
			if( gameInfo != null )
			{
				startBrowser( gameInfo, LauncherExceptionCode.ERR_CANNOT_VIEW_GAMEINFO );
			}
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onClickMenuHelpAbout()
	 */
	@Override
	public void onRequestAboutScreen()
	{
		view.displayAbout( currentLanguage, Language.isRightToLeft( currentLanguage ) );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onMoveGames(java.util.Set, java.lang.String)
	 */
	@Override
	public Set<String> onRequestMoveGamesScreen( Set<String> gameNames, String oldDatabase )
	{
		//create a list of databases that excludes the current one
		Set<String> targetDatabases = new HashSet<String>( databases );
		targetDatabases.remove( currentDatabase );

		return view.displayAndGetMoveGames( currentLanguage, gameNames, oldDatabase, targetDatabases,
				Language.isRightToLeft( currentLanguage ) );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onMoveGamesOK(java.util.Set, java.lang.String, java.lang.String)
	 */
	@Override
	public Set<String> onRequestMoveGamesAction( Set<String> gameNames, String oldDatabase, String newDatabase )
			 throws LauncherException
	{
		//create a Game set from the game names
		Set<Game> games = new HashSet<Game>();

		for( String gameName: gameNames )
		{
			games.add( gamesMap.get( gameName ) );
		}

		Set<String> movedGameNames = new HashSet<String>();

		try
		{
			Set<Game> movedGames = gamePersister.moveGames( games, oldDatabase, newDatabase,
					new MoveGameActionDecider( view, currentLanguage, Language.isRightToLeft( currentLanguage ) ) );

			for( Game movedGame: movedGames )
			{
				movedGameNames.add( movedGame.getName() );
				gamesMap.remove( movedGame.getName() );
			}
		}
		catch( GameNotFoundException gnfe )
		{
			//don't have the game name
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_NOT_FOUND );
		}
		catch( DatabaseNotFoundException dnfe )
		{
			//don't know which one was not found
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND );
		}
		catch( GameWithNullNameException gwnne )
		{
			//don't have the game with null name
			throw new LauncherException( LauncherExceptionCode.ERR_GAME_WITH_NULL_NAME );
		}
		catch( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}

		view.updateGameCount( gamesMap.size() );

		return movedGameNames;
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onClickMenuActionsCreateEmptyDatabase()
	 */
	@Override
	public void onRequestCreateEmptyDatabaseScreen()
	{
		view.displayCreateEmptyDatabase( currentLanguage, Language.isRightToLeft( currentLanguage ) );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onRequestCreateEmptyDatabase(java.lang.String)
	 */
	@Override
	public void onRequestCreateEmptyDatabaseAction( String newDatabase ) throws LauncherException
	{
		try
		{
			gamePersister.createDatabase( newDatabase );

			view.addDatabase( newDatabase );
			databases.add( newDatabase );
		}
		catch( DatabaseNullNameException dnne )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NULL_NAME );
		}
		catch( DatabaseAlreadyExistsException daee )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_ALREADY_EXISTS, newDatabase );
		}
		catch( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onRequestHelpFile()
	 */
	@Override
	public void onRequestHelpFile() throws LauncherException
	{
		String helpFileFullPath = new File( helpFileDirectory, "README.html" ).getAbsolutePath();
		startBrowser( helpFileFullPath, LauncherExceptionCode.ERR_CANNOT_VIEW_HELP_FILE );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter#onRequestLocateFile(java.lang.String)
	 */
	@Override
	public void onRequestLocateFile( String gameName ) throws LauncherException
	{
		Game game = gamesMap.get( gameName );

		File file = new File(FileTypeUtils.getMainFile( game.getRomA(),
				game.getRomB(),
				game.getDiskA(),
				game.getDiskB(),
				game.getTape(),
				game.getHarddisk(),
				game.getLaserdisc(),
				game.getTclScript()));

		try
		{
			fileLocator.locateFile( file.getAbsolutePath() );
		}
		catch( IOException e )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_CANNOT_LOCATE_FILE );
		}
	}

	private void startBrowser( String uriString, LauncherExceptionCode errCodeIfNotFound ) throws LauncherException
	{
		URI uri;

		//first inspect the uriString string to see if it is a file or URL
		File file = new File( uriString );
		if( file.exists() )
		{
			uri = file.toURI();
		}
		else
		{
			//then this is not a file - assume it is a URL
			try
			{
				uri = new URI( uriString );
			}
			catch( URISyntaxException e )
			{
				throw new LauncherException( errCodeIfNotFound );
			}
		}

		//now bring up the default browser of the system for URL or the system default application
		Desktop desktop = Desktop.isDesktopSupported() ? Desktop.getDesktop() : null;
	    if( desktop != null && desktop.isSupported( Desktop.Action.OPEN ) )
	    {
	    	try
	    	{
				desktop.browse( uri );
			}
	    	catch( IOException e )
	    	{
				throw new LauncherException( LauncherExceptionCode.ERR_IO );
			}
	    }
	    else
	    {
	    	//the following link:
	    	// http://stackoverflow.com/questions/8258153/how-to-get-desktop-class-supported-under-linux
	    	//explains why code might get here on Linux
			throw new LauncherException( errCodeIfNotFound );
	    }
	}

	private void setLanguageParameters( Language language )
	{
		currentLanguage = language;
		if( currentLanguage == null )
		{
			//in this case try to get system default
			Locale locale = Locale.getDefault();
			currentLanguage = Language.fromLocale( locale.getLanguage() + "_" + locale.getCountry() );

			//if still cannot find this locale, then force English
			if( currentLanguage == null )
			{
				currentLanguage = Language.ENGLISH;
			}

			currentLanguageCode = DEFAULT_SYSTEM_LANGUAGE;
		}
		else
		{
			currentLanguageCode = currentLanguage.toString();
		}

		currentRightToLeft = Language.isRightToLeft( currentLanguage );		
	}

	private Set<GameLabel> getSortedGameList()
	{
		Set<GameLabel> sortedList = new LinkedHashSet<GameLabel>();
		if( gamesMap != null )
		{
			Game[] games = gamesMap.values().toArray( new Game[0] );

			Arrays.sort( games, new Comparator<Game>() {
				@Override
					public int compare( Game game1, Game game2 )
					{
						return game1.getName().compareToIgnoreCase( game2.getName() );
					}
			});

			for( Game game: games )
			{
				sortedList.add( new GameLabel( game.getName(), getGameMedium( game ) ) );
			}
		}
		
		return sortedList;
	}

	private Medium getGameMedium( Game game )
	{
		Medium medium = null;
		
		if( game.isROM() )
		{
			medium = Medium.ROM;
		}
		else if( game.isDisk() )
		{
			medium = Medium.DISK;
		}
		else if( game.isHarddisk() )
		{
			medium = Medium.HARDDISK;
		}
		else if( game.isTape() )
		{
			medium = Medium.TAPE;
		}
		else if( game.isLaserdisc() )
		{
			medium = Medium.LASERDISC;
		}
		else if( game.isScript() )
		{
			medium = Medium.SCRIPT;
		}

		return medium;
	}

	private void retrieveDatabaseGames( String database ) throws DatabaseNotFoundException, IOException
	{
		Set<Game> games =  gamePersister.getGames( database );
		setGameMap( games );
	}

	private void setGameMap( Set<Game> games )
	{
		gamesMap = new HashMap<String,Game>();

		for( Game game: games )
		{
			gamesMap.put( game.getName(), game );
		}
	}

	private Game getGameObject( String name,
			String info,
			String machine,
			String romA,
			String romB,
			String extensionRom,
			String diskA,
			String diskB,
			String tape,
			String harddisk,
			String laserdisc,
			String script)
	{
		Game game = null;

		String mainFile = FileTypeUtils.getMainFile( romA, romB, diskA, diskB, tape, harddisk, laserdisc, script );

		if( mainFile == null )
		{
			throw new IllegalArgumentException();
		}

		int msxGenID = 0;
		boolean isMSX = false;
		boolean isMSX2 = false;
		boolean isMSX2Plus = false;
		boolean isTurboR = false;
		boolean isPSG = false;
		boolean isSCC = false;
		boolean isSCCI = false;
		boolean isPCM = false;
		boolean isMSXMUSIC = false;
		boolean isMSXAUDIO = false;
		boolean isMoonsound = false;
		boolean isMIDI = false;
		String screenshotSuffix = null;

		String sha1Code = null;
		long fileSize = 0;
		boolean notScript = isNotScript( romA, romB, diskA, diskB, tape, harddisk, laserdisc, script );

		if( notScript )
		{
			File file = new File( mainFile );
			if( file.exists() )
			{
				if( FileTypeUtils.isZIP( file ) )
				{
					ZipFile zip = null;
					InputStream inputStream = null;
	
					try
					{
						zip = new ZipFile( file );
	
						ZipEntry firstZipEntry = zip.entries().nextElement();
	
						inputStream = zip.getInputStream( firstZipEntry );
						sha1Code = HashUtils.getSHA1Code( inputStream );
						fileSize = firstZipEntry.getSize();
					}
					catch( ZipException e )
					{
						//invalid ZIP file - skip and return null
					}
					catch( IOException e )
					{
						//shouldn't happen here
					}
					finally
					{
						IOUtils.forceClose( inputStream );
						IOUtils.forceClose( zip );
					}			
				}
				else
				{
					fileSize = file.length();
					sha1Code = HashUtils.getSHA1Code( file );
				}
	
			}
	
			ExtraData extraData = extraDataMap.get( sha1Code );
			if( extraData != null )
			{
				msxGenID = extraData.getMSXGenerationsID();
				isMSX = extraData.isMSX();
				isMSX2 = extraData.isMSX();
				isMSX2Plus = extraData.isMSX();
				isTurboR = extraData.isMSX();
				isPSG = extraData.isPSG();
				isSCC = extraData.isSCC();
				isSCCI = extraData.isSCCI();
				isPCM = extraData.isPCM();
				isMSXMUSIC = extraData.isMSXMUSIC();
				isMSXAUDIO = extraData.isMSXAUDIO();
				isMoonsound = extraData.isMoonsound();
				isMIDI = extraData.isMIDI();
				screenshotSuffix = extraData.getSuffix();
			}
		}

		game = Game.name( name )
				.info( info )
				.machine( machine )
				.romA( romA )
				.romB( romB )
				.extensionRom( extensionRom )
				.diskA( diskA )
				.diskB( diskB )
				.tape( tape )
				.harddisk( harddisk )
				.laserdisc( laserdisc )
				.tclScript( script )
				.sha1Code( sha1Code )
				.size( fileSize )
				.msxGenID( msxGenID )
				.isMSX( isMSX )
				.isMSX2( isMSX2 )
				.isMSX2Plus( isMSX2Plus )
				.isTurboR( isTurboR )
				.isPSG( isPSG )
				.isSCC( isSCC )
				.isSCCI( isSCCI )
				.isPCM( isPCM )
				.isMSXMUSIC( isMSXMUSIC )
				.isMSXAUDIO( isMSXAUDIO )
				.isMoonsound( isMoonsound )
				.isMIDI( isMIDI )
				.screenshotSuffix( screenshotSuffix )
				.build();

		return game;
	}

	private boolean isNotScript( String romA, String romB, String diskA, String diskB,
			String tape, String harddisk, String laserdisc, String script )
	{
		return !Utils.isEmpty( romA ) || !Utils.isEmpty( romB ) ||
				!Utils.isEmpty( diskA ) || !Utils.isEmpty( diskB ) ||
				!Utils.isEmpty( tape ) || !Utils.isEmpty( harddisk ) || !Utils.isEmpty( laserdisc ) ||
				Utils.isEmpty( script );
	}

	private void initializeRepositoryInfoMap()
	{
		try
		{
			repositoryInfoMap = repositoryData.getRepositoryInfo();
		}
		catch( Exception e )
		{
			//in this case reset it
			repositoryInfoMap = null;
		}
	}

	//TODO move this to Utils class and write a test for it
	private boolean equalStrings( String str1, String str2 )
	{
		return (str1 == null ? str2 == null : str1.equals( str2 ));
	}
}
