/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.view;

import info.msxlaunchers.openmsx.launcher.data.global.Settings;
import info.msxlaunchers.openmsx.launcher.data.global.constants.Language;
import info.msxlaunchers.openmsx.launcher.ui.presenter.GameLabel;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherPresenter;

import java.util.Set;

/**
 * Interface for UI View
 * 
 * @since v1.0
 * @author Sam Elsharif
 *
 */
public interface LauncherView
{
	/**
	 * Sets reference to Presenter
	 * 
	 * @param presenter Reference to Presenter
	 */
	void setPresenter( LauncherPresenter presenter );

	/**
	 * Displays main screen of UI
	 * 
	 * @param language Language
	 * @param games Set containing GameLabel objects
	 * @param databases Set containing database names
	 * @param defaultDatabase Name of default database to display - could be null
	 * @param rightToLeft Flag to determine screen orientation based on language
	 */
	void displayMain( Language language,
			Set<GameLabel> games,
			Set<String> databases,
			String defaultDatabase,
			boolean rightToLeft );

	/**
	 * Displays settings screen
	 * 
	 * @param settings
	 * @param databases
	 * @param language Language
	 * @param languageCode
	 * @param rightToLeft Flag to determine screen orientation based on language
	 */
	void displaySettings( Settings settings,
			Set<String> databases,
			Language language,
			String languageCode,
			boolean rightToLeft );

	/**
	 * Refreshes language display in the application based on the given language
	 * 
	 * @param language Language
	 */
	void refreshLanguage( Language language );

	/**
	 * Flips screen orientation to be left to right (for languages such as English)
	 */
	void flipOrientationLeftToRight();

	/**
	 * Flips screen orientation to be right to left (for languages such as Arabic)
	 */
	void flipOrientationRightToLeft();

	/**
	 * Fills the game list with given games
	 * 
	 * @param currentDatabase Current database name
	 * @param games Set containing GameLabel objects
	 * @param selectedGame
	 */
	void fillGameList( String currentDatabase, Set<GameLabel> games, String selectedGame );

	/**
	 * Updates the total game count on the main screen
	 * 
	 * @param total Total game count of the current database
	 */
	void updateGameCount( int total );

	/**
	 * Displays screen to add a new game
	 * 
	 * @param language Language
	 * @param machines Set containing list of openMSX machines
	 * @param extensions Set containing list of openMSX extensions
	 * @param rightToLeft Flag to determine screen orientation based on language
	 */
	void displayAddEdit( Language language, Set<String> machines, Set<String> extensions, boolean rightToLeft );

	/**
	 * Displays screen to edit an existing game
	 * 
	 * @param language Language
	 * @param machines Set containing list of openMSX machines
	 * @param extensions Set containing list of openMSX extensions
	 * @param rightToLeft Flag to determine screen orientation based on language
	 * @param name Name of game
	 * @param info Info file or URL for a game
	 * @param machine Machine name
	 * @param romA Full path to ROM A file
	 * @param romB Full path to ROM B file
	 * @param extension Name of openMSX extension
	 * @param diskA Full path to Disk A file
	 * @param diskB Full path to Disk B file
	 * @param tape Full path to Tape file
	 * @param harddisk Full path to Hard Disk file
	 * @param laserdisc Full path to Laserdisc file
	 * @param script Full path to Script file
	 */
	void displayAddEdit( Language language, Set<String> machines, Set<String> extensions, boolean rightToLeft, 
			String name, String info, String machine, String romA, String romB, String extension,
			String diskA, String diskB, String tape, String harddisk, String laserdisc, String script );

	/**
	 * Displays screen to fill a database with games by scanning folders on disks
	 * 
	 * @param language Language
	 * @param databases Set containing current list of existing databases
	 * @param currentDatabase Name of current database
	 * @param machines Set containing list of openMSX machines
	 * @param rightToLeft Flag to determine screen orientation based on language
	 */
	void displayFillDatabase( Language language,
			Set<String> databases,
			String currentDatabase,
			Set<String> machines,
			boolean rightToLeft );

	/**
	 * Adds database to the list of database names in the screen
	 * 
	 * @param database Database name
	 */
	void addDatabase( String database );

	/**
	 * Displays About screen
	 * 
	 * @param language Language
	 * @param rightToLeft Flag to determine screen orientation based on language
	 */
	void displayAbout( Language language, boolean rightToLeft );

	/**
	 * Shows the given two image files
	 * 
	 * @param screenshot1 Full path to screenshot1 image file - could be null
	 * @param screenshot2 Full path to screenshot2 image file - could be null
	 */
	void showGameScreenshots( String screenshot1, String screenshot2 );

	/**
	 * Enables or disables the four buttons: Launch, Remove, Edit and Info based on the given flags
	 * 
	 * @param launchFlag True if to enable, false to disable
	 * @param removeFlag True if to enable, false to disable
	 * @param editFlag True if to enable, false to disable
	 * @param infoFlag True if to enable, false to disable
	 */
	void enableButtons( boolean launchFlag, boolean removeFlag, boolean editFlag, boolean infoFlag );

	/**
	 * Displays screen to ask user for the target database and return a set of the moved game names
	 * 
	 * @param language Language
	 * @param gameNames Set containing game names to move
	 * @param oldDatabase Name of current database to move from
	 * @param targetDatabase  Name of database to move to
	 * @param rightToLeft Flag to determine screen orientation based on language
	 * @return Set of moved game names - null if operation was cancelled
	 */
	Set<String> displayAndGetMoveGames( Language language,
			Set<String> gameNames,
			String oldDatabase,
			Set<String> targetDatabase,
			boolean rightToLeft );

	/**
	 * Displays Action Decider screen to ask the user to decide how to proceed
	 * 
	 * @param gameName Name of game to show in the message
	 * @param language Language
	 * @param rightToLeft Flag to determine screen orientation based on language
	 * @return A 0-based integer that represents the desired action, or -1 to mean that the decision operation was cancelled.
	 */
	int displayAndGetActionDecider( String gameName, Language language, boolean rightToLeft );

	/**
	 * Displays screen to ask the user for the name of the empty database to create
	 * 
	 * @param language Language
	 * @param rightToLeft Flag to determine screen orientation based on language
	 */
	void displayCreateEmptyDatabase( Language language, boolean rightToLeft );

	/**
	 * Shows company name, year and size of main file for selected game
	 * 
	 * @param company Name of company
	 * @param year Year of game release
	 * @param size Size of main game file in KB
	 */
	void showGameCompanyYearSizeData( String company, String year, long size );

	/**
	 * Resets display of company name, year and size of main file
	 */
	void resetGameCompanyYearSizeData();
}
